@@ -5,11 +5,14 @@ from .agent import (
     # create_planner_agent,
     get_model_name,
 )
+from .approval_hooks import ApprovalHooks, ToolApprovalError
 from .hooks import ToolDisplayHooks, get_display_hooks
 
 __all__ = [
     "ToolDisplayHooks",
     "get_display_hooks",
+    "ApprovalHooks",
+    "ToolApprovalError",
     # "create_planner_agent",
     "create_dev_agent",
     "get_model_name",
