@@ -67,8 +67,20 @@ class ClearCommand(SlashCommand):
 
     async def execute(self, scheduler, *_args) -> str:
         """Execute the /clear command."""
-        await scheduler.context_manager.clear()
-        return "🧹 Context Cleared"
+        from ..utils.sessions import default_session_local_ms
+
+        new_sid = default_session_local_ms()
+
+        # Clear screen
+        try:
+            import sys as _sys
+
+            _sys.stdout.write("\033[2J\033[H")
+            _sys.stdout.flush()
+        except Exception:
+            pass
+
+        return f"session_switch:{new_sid}"  # Special return value for CLI to handle
 
 
 class StatusCommand(SlashCommand):
@@ -105,20 +117,6 @@ class StatusCommand(SlashCommand):
         table.add_row("Total Sessions", str(len(sessions)))
         table.add_row("Working Directory", os.getcwd())
 
-        # Show environment variables
-        env_vars = {
-            "OPENAI_API_KEY": "✅ Set" if os.environ.get("OPENAI_API_KEY") else "❌ Not set",
-            "AZURE_OPENAI_API_KEY": (
-                "✅ Set" if os.environ.get("AZURE_OPENAI_API_KEY") else "❌ Not set"
-            ),
-            "KODER_MODEL": os.environ.get("KODER_MODEL", "Not set (using default)"),
-        }
-
-        table.add_section()
-        table.add_row("[bold]Environment[/bold]", "[bold]Status[/bold]")
-        for key, value in env_vars.items():
-            table.add_row(key, value)
-
         if sessions:
             table.add_section()
             table.add_row("[bold]Available Sessions[/bold]", "[bold]ID[/bold]")
@@ -175,15 +173,62 @@ class McpCommand(SlashCommand):
             return f"Error listing MCP servers: {e}"
 
 
+class HelpCommand(SlashCommand):
+    """Show help and usage instructions."""
+
+    def __init__(self):
+        super().__init__("help", "Show help and usage instructions")
+
+    async def execute(self, scheduler, *_args) -> str:
+        """Execute the /help command."""
+        # Show general usage instructions
+        usage_text = """[bold cyan]Koder - AI Coding Assistant[/bold cyan]
+
+[bold]How to use:[/bold]
+• Type your coding questions or requests in natural language
+• Use slash commands (/) for special functions
+• Press Ctrl+C to exit interactive mode
+• Type 'exit' or 'quit' to end the session
+
+[bold]Examples:[/bold]
+• "Help me implement a login feature"
+• "Fix the bug in auth.py"
+• "Explain this code"
+• "/status" - show current configuration
+• "/clear" - start a new session"""
+
+        console.print(Panel(usage_text, title="📖 Help", border_style="cyan"))
+
+        # Get the command handler to access all commands
+        handler = scheduler.slash_handler if hasattr(scheduler, "slash_handler") else slash_handler
+        commands = handler.get_command_list()
+
+        if commands:
+            # Show available slash commands in a table
+            table = Table(show_header=True, header_style="bold cyan", box=None)
+            table.add_column("Command", style="cyan", width=12)
+            table.add_column("Description", style="white")
+
+            for name, description in commands:
+                table.add_row(f"/{name}", description)
+
+            console.print()
+            console.print(table)
+
+        return ""
+
+
 class SlashCommandHandler:
     """Handles slash command detection and execution."""
 
     def __init__(self):
         self.commands: Dict[str, SlashCommand] = {
+            "help": HelpCommand(),
             "init": InitCommand(),
             "clear": ClearCommand(),
             "status": StatusCommand(),
             "mcp": McpCommand(),
+            "session": SlashCommand("session", "Switch session via picker and recreate scheduler"),
         }
 
     def get_command_list(self) -> List[Tuple[str, str]]:
@@ -213,6 +258,20 @@ class SlashCommandHandler:
 
         try:
             command = self.commands[command_name]
+            if command_name == "session":
+                from ..utils.sessions import picker_arrows, sort_sessions_desc
+
+                # Get sessions and prompt for selection
+                sessions = await scheduler.context_manager.list_sessions()
+                if not sessions:
+                    return "No sessions found."
+
+                sessions = sort_sessions_desc(sessions)
+                selected = picker_arrows(sessions)
+
+                if selected:
+                    return f"session_switch:{selected}"  # Special return value for CLI to handle
+                return "Session switch cancelled"
             return await command.execute(scheduler, *args)
         except Exception as e:
             return f"❌ Error executing command '{command_name}': {str(e)}"
