@@ -92,10 +92,10 @@ class InteractivePrompt:
         self.session = PromptSession(
             completer=self.completer,
             complete_while_typing=True,
-            mouse_support=True,
+            mouse_support=False,
         )
 
-    async def get_input(self, prompt_text: str = "user") -> str:
+    async def get_input(self, prompt_text: str = "User") -> str:
         """Get user input with Rich panel display and prompt_toolkit completion."""
         # Create buffer
         buffer = Buffer(
@@ -120,7 +120,7 @@ class InteractivePrompt:
         # Create simple frame without heavy styling
         framed_input = Frame(
             body=input_window,
-            title=f"{prompt_text}",
+            title=f"{prompt_text} (Ctrl+C to exit)",
         )
 
         # Key bindings
@@ -224,7 +224,7 @@ class InteractivePrompt:
             key_bindings=kb,
             style=style,
             full_screen=False,
-            mouse_support=True,
+            mouse_support=False,  # Disable mouse support to allow terminal scrolling
         )
 
         result = await app.run_async()
