@@ -17,8 +17,9 @@ from rich.console import Console, Group
 from rich.live import Live
 from rich.text import Text
 
-from ..agentic import create_dev_agent, get_display_hooks
+from ..agentic import ApprovalHooks, ToolApprovalError, create_dev_agent, get_display_hooks
 from ..core.context import ContextManager
+from ..core.permissions import PermissionManager
 from ..core.streaming_display import StreamingDisplayManager
 from ..tools import get_all_tools
 
@@ -31,10 +32,13 @@ class AgentScheduler:
     def __init__(self, session_id: str = "default", streaming: bool = False):
         self.semaphore = asyncio.Semaphore(10)
         self.context_manager = ContextManager(session_id)
+        self.permission_manager = PermissionManager()
         self.tools = get_all_tools()
         self.dev_agent = None  # Will be initialized in async method
         self.streaming = streaming
-        self.hooks = get_display_hooks(streaming_mode=streaming)
+        # Create approval hooks that wrap display hooks
+        display_hooks = get_display_hooks(streaming_mode=streaming)
+        self.hooks = ApprovalHooks(self.permission_manager, display_hooks)
         self._agent_initialized = False
         self._mcp_servers = []  # Track MCP servers for cleanup
 
@@ -66,7 +70,7 @@ class AgentScheduler:
             self.dev_agent = await create_dev_agent(self.tools)
             # Track MCP servers for cleanup
             if hasattr(self.dev_agent, "mcp_servers") and self.dev_agent.mcp_servers:
-                self._mcp_servers = self.dev_agent.mcp_servers
+                self._mcp_servers = list(self.dev_agent.mcp_servers)  # Create a copy
             self._agent_initialized = True
 
     async def handle(self, user_input: str) -> str:
@@ -100,23 +104,31 @@ class AgentScheduler:
 
         # Run the agent
         async with self.semaphore:
-            if self.streaming:
-                response = await self._handle_streaming(full_input)
-            else:
-                result = await Runner.run(
-                    self.dev_agent,
-                    full_input,
-                    run_config=RunConfig(),
-                    hooks=self.hooks,
-                    max_turns=50,
-                )
-                # Filter output for security
-                response = self._filter_output(result.final_output)
+            try:
+                if self.streaming:
+                    response = await self._handle_streaming(full_input)
+                else:
+                    result = await Runner.run(
+                        self.dev_agent,
+                        full_input,
+                        run_config=RunConfig(),
+                        hooks=self.hooks,
+                        max_turns=50,
+                    )
+                    # Filter output for security
+                    response = self._filter_output(result.final_output)
 
-                # Clean response output without heavy panels
-                print()  # Add space before response
+                    # Clean response output without heavy panels
+                    print()  # Add space before response
+                    console.print(response)
+                    print()  # Add space after response
+            except ToolApprovalError as e:
+                # Handle tool denial gracefully
+                response = (
+                    f"[red]Execution stopped: {str(e)}[/red]\n\nPlease provide new instructions."
+                )
                 console.print(response)
-                print()  # Add space after response
+                return response
 
         # Save conversation to context
         await self.context_manager.save(
@@ -144,10 +156,8 @@ class AgentScheduler:
         # Add space before streaming starts
         print()
 
-        # Capture cursor position before Rich Live starts (if supported)
-        if supports_advanced_clearing:
-            sys.stdout.write("\033[6n")  # Request cursor position
-            sys.stdout.flush()
+        # Skip cursor position capture - it causes escape sequence leaks
+        # The terminal clearing strategies below work without explicit position tracking
 
         # Run the agent in streaming mode
         if self.dev_agent is None:
@@ -169,9 +179,9 @@ class AgentScheduler:
         with Live(
             "",
             console=console,
-            refresh_per_second=8,  # Reasonable refresh rate
-            transient=False,  # Keep content visible during streaming
-            vertical_overflow="visible",  # Show all content even if it exceeds screen height
+            refresh_per_second=8,
+            transient=True,
+            vertical_overflow="crop",
         ) as live:
             try:
                 # Process streaming events
@@ -262,9 +272,17 @@ class AgentScheduler:
                         console.print(f"[dim red]Event processing error: {e}[/dim red]")
                         continue
 
+            except ToolApprovalError as e:
+                # Handle tool denial in streaming mode
+                error_msg = f"Execution stopped: {str(e)}"
+                console.print(f"[red]{error_msg}[/red]")
+                # Clear the display manager and return early
+                display_manager.finalize_text_sections()
+                return f"{error_msg}\n\nPlease provide new instructions."
             except Exception as e:
                 # Handle streaming errors
                 console.print(f"[dim red]Streaming error: {e}[/dim red]")
+                # Continue to try to get some output
 
         # After Rich Live context ends, perform intelligent cleanup
         display_manager.finalize_text_sections()
@@ -280,7 +298,7 @@ class AgentScheduler:
             # Strategy 1: For advanced terminals, clear the scroll buffer region
             if supports_advanced_clearing:
                 try:
-                    # Clear recent lines from scroll buffer (terminal-specific)
+                    # Clear recent lines from scrollback (terminal-specific)
                     if terminal_type == "iTerm.app":
                         # iTerm2 specific: Clear last N lines from scrollback
                         lines_count = self._get_line_count(final_content)
@@ -293,10 +311,17 @@ class AgentScheduler:
                 except Exception:
                     pass  # Fallback to simple approach
 
-            # Strategy 2: Print final clean content only if not already displayed
-            # Skip if we already showed content during streaming
-            if not content_displayed:
-                # No content was displayed during streaming, show it now
+            # Strategy 2: Always show final text response, but avoid duplicate tool output
+            # Get the final text response separately to ensure it's always displayed
+            final_text_response = display_manager.get_final_text()
+
+            if final_text_response and final_text_response.strip():
+                # Always show the final AI text response
+                print()  # Add spacing
+                console.print(final_text_response)
+                print()  # Add spacing after
+            elif not content_displayed:
+                # Fallback: show full content if no separate text and nothing was displayed
                 print()  # Add spacing
                 console.print(final_content)
                 print()  # Add spacing after
@@ -337,6 +362,12 @@ class AgentScheduler:
         """Filter sensitive information from output."""
         import re
 
+        # Handle None or non-string input
+        if text is None:
+            return ""
+        if not isinstance(text, str):
+            text = str(text)
+
         # Filter API keys and tokens
         text = re.sub(r"sk-\w{10,}", "[TOKEN]", text)
         text = re.sub(
@@ -346,10 +377,34 @@ class AgentScheduler:
 
     async def cleanup(self):
         """Clean up resources, including MCP servers."""
-        if self._mcp_servers:
-            for server in self._mcp_servers:
-                try:
-                    await server.cleanup()
-                except Exception as e:
-                    console.print(f"[dim red]Error disconnecting MCP server: {e}[/dim red]")
-            self._mcp_servers.clear()
+        try:
+            # Clean up MCP servers one by one to avoid task group issues
+            if self._mcp_servers:
+                for server in self._mcp_servers:
+                    try:
+                        if hasattr(server, "cleanup"):
+                            # Try cleanup with a timeout to avoid hanging
+                            try:
+                                await asyncio.wait_for(server.cleanup(), timeout=3.0)
+                            except asyncio.TimeoutError:
+                                console.print(
+                                    f"[dim red]MCP server {getattr(server, 'name', 'unknown')} cleanup timed out[/dim red]"
+                                )
+                            except Exception as cleanup_error:
+                                console.print(
+                                    f"[dim red]Error cleaning up MCP server {getattr(server, 'name', 'unknown')}: {cleanup_error}[/dim red]"
+                                )
+                    except Exception as e:
+                        console.print(
+                            f"[dim red]Error accessing MCP server for cleanup: {e}[/dim red]"
+                        )
+
+                self._mcp_servers.clear()
+
+            # Reset agent state to force re-initialization
+            if self.dev_agent:
+                self.dev_agent = None
+                self._agent_initialized = False
+
+        except Exception as e:
+            console.print(f"[dim red]Unexpected error during scheduler cleanup: {e}[/dim red]")
